@@ -13,11 +13,19 @@ python -c "import thapbi_pict; print('Direct import says version ' + thapbi_pict
 thapbi_pict -v
 python -m thapbi_pict -v
 
+if [ ! -f tests/run_tests.sh ]; then
+    echo "Please run tests/run_tests.sh from the top level directory"
+    false
+fi
+
+time tests/test_build_db.sh
+
 time tests/test_woody_hosts.sh
 
+time tests/test_ena-submit.sh
 time tests/test_dump.sh
 time tests/test_load-tax.sh
-time tests/test_legacy-import.sh
+time tests/test_curated-import.sh
 time tests/test_conflicts.sh
 
 if ! [ -x "$(command -v cutadapt)" ]; then
@@ -33,14 +41,15 @@ elif ! [ -x "$(command -v cutadapt)" ]; then
     echo 'WARNING: cutadapt not installed, skipping some tests'
 else
     time tests/test_prepare-reads.sh
+    time tests/test_synthetic_controls.sh
     time tests/test_pipeline.sh
 fi
 
+time tests/test_fasta-nr.sh
 time tests/test_classify.sh
-time tests/test_seq-import.sh
+time tests/test_curated-seq.sh
 time tests/test_assess.sh
-time tests/test_sample-summary.sh
-time tests/test_read-summary.sh
+time tests/test_summary.sh
 time tests/test_edit-graph.sh
 
 echo "================="
