@@ -1,6 +1,6 @@
 #!/bin/bash
 
-# Copyright 2019 by Peter Cock, The James Hutton Institute.
+# Copyright 2019-2021 by Peter Cock, The James Hutton Institute.
 # All rights reserved.
 # This file is part of the THAPBI Phytophthora ITS1 Classifier Tool (PICT),
 # and is released under the "MIT License Agreement". Please see the LICENSE
@@ -10,19 +10,25 @@ IFS=$'\n\t'
 set -eu
 set -o pipefail
 
-export TMP=${TMP:-/tmp}
-
 # Note all tests here (initially) using default database:
 
+export TMP=${TMP:-/tmp/thapbi_pict}/conflicts
+mkdir -p $TMP
+
 echo "=================="
 echo "Checking conflicts"
 echo "=================="
+set -x
 set -o pipefail
 
 thapbi_pict conflicts -o $TMP/conflicts.tsv
 diff $TMP/conflicts.tsv tests/conflicts/default.tsv
 
-if [ ! -f $TMP/dup_seqs.sqlite ]; then echo "Run tests/test_legacy-import.sh to setup test DB"; false; fi
+# Same test is used in tests/test_curated-import.sh
+# Only 6 FASTA records, but two are double entries so want 8 here
+export DB=$TMP/dup_seqs.sqlite
+rm -rf $DB
+thapbi_pict import -x -d $DB -i tests/curated-import/dup_seqs.fasta -c ncbi -s $'\001'
 thapbi_pict conflicts -d $TMP/dup_seqs.sqlite -o $TMP/dup_seqs.tsv
 diff $TMP/dup_seqs.tsv tests/conflicts/dup_seqs.tsv
 
