@@ -1,34 +1,33 @@
-# Copyright 2019 by Peter Cock, The James Hutton Institute.
+# Copyright 2019-2021 by Peter Cock, The James Hutton Institute.
 # All rights reserved.
 # This file is part of the THAPBI Phytophthora ITS1 Classifier Tool (PICT),
 # and is released under the "MIT License Agreement". Please see the LICENSE
 # file that should have been included as part of this package.
-
 """Generate edit-distance network graph from FASTA files.
 
 This implements the ``thapbi_pict edit-graph ...`` command.
 """
-
-import os
 import sys
-
 from collections import Counter
 
-import numpy as np
 import matplotlib.pyplot as plt
 import networkx as nx
-
-from Levenshtein import distance as levenshtein
-
-from sqlalchemy.orm import aliased, contains_eager
-
+import numpy as np
 from Bio.SeqIO.FastaIO import SimpleFastaParser
+from Levenshtein import distance as levenshtein
+from sqlalchemy.orm import aliased
+from sqlalchemy.orm import contains_eager
 
-from .db_orm import ITS1, SequenceSource, Taxonomy, connect_to_db
-
+from .db_orm import connect_to_db
+from .db_orm import RefMarker
+from .db_orm import SequenceSource
+from .db_orm import Taxonomy
+from .utils import file_to_sample_name
+from .utils import find_paired_files
 from .utils import find_requested_files
 from .utils import genus_species_name
 from .utils import md5seq
+from .utils import parse_species_tsv
 from .utils import species_level
 from .utils import split_read_name_abundance
 
@@ -36,6 +35,9 @@ from .utils import split_read_name_abundance
 genus_color = {
     # From the VGA colors, in order of DB abundance,
     # dark red for other, grey for none, dark orange for conflicts
+    #
+    # Oomycetes; Peronosporales:
+    # ==========================
     "Phytophthora": "#FF0000",  # Red
     "Peronospora": "#00FF00",  # Lime
     "Hyaloperonospora": "#0000FF",  # Blue
@@ -50,6 +52,14 @@ genus_color = {
     # Basidiophora
     # Calycofera
     # Plasmoverna
+    #
+    # Nematoda:
+    # =========
+    "Globodera": "#FF0000",  # Red
+    "Heterodera": "#00FF00",  # Lime
+    #
+    # Special cases:
+    # ==============
     "synthetic": "#FFA500",  # Orange
 }
 
@@ -58,13 +68,13 @@ def write_pdf(G, filename):
     """Render NetworkX graph to PDF using GraphViz fdp."""
     # TODO: Try "sfdp" but need GraphViz built with triangulation library
     default = G.graph["node_default"]["color"]
-    node_colors = [G.node[_].get("color", default) for _ in G]
+    node_colors = [G.nodes[_].get("color", default) for _ in G]
 
     default = 1.0
-    node_sizes = [G.node[_].get("size", default) for _ in G]
+    node_sizes = [G.nodes[_].get("size", default) for _ in G]
 
     default = ""
-    node_labels = {_: G.node[_].get("label", default) for _ in G}
+    node_labels = {_: G.nodes[_].get("label", default) for _ in G}
 
     default = G.graph["edge_default"]["style"]
     edge_styles = [G.edges[_].get("style", default) for _ in G.edges()]
@@ -115,25 +125,33 @@ def write_xgmml(G, filename, name="THAPBI PICT edit-graph"):
             if not label:
                 label = "{%s}" % n[:6]  # start of MD5, prefixed for sorting
             # weight = node["weight"]
-            handle.write('  <node id="%s" label="%s">\n' % (n, label))
+            handle.write(f'  <node id="{n}" label="{label}">\n')
             color = node["color"]
             # Size 1 to 100 works fine in PDF output, not so good in Cytoscape!
             # Rescale to use range 5 to 50.
             size = (node["size"] * 0.45) + 5.0
             handle.write(
-                '    <graphics type="CIRCLE" fill="%s" outline="#000000" '
-                'h="%0.2f" w="%0.2f"/>\n' % (color, size, size)
+                f'    <graphics type="CIRCLE" fill="{color}" outline="#000000" '
+                'h="%0.2f" w="%0.2f"/>\n' % (size, size)
             )
             # Cytoscape hides the node ID (presumably assumes not usually user facing):
-            handle.write('    <att type="string" name="MD5" value="%s"/>\n' % n)
+            handle.write(f'    <att type="string" name="MD5" value="{n}"/>\n')
             handle.write(
                 '    <att type="integer" name="Total-abundance" value="%i"/>\n'
-                % node.get("abundance", 0)
+                % node.get("total_abundance", 0)
+            )
+            handle.write(
+                '    <att type="integer" name="Max-sample-abundance" value="%i"/>\n'
+                % node.get("max_sample_abundance", 0)
             )
             handle.write(
                 '    <att type="integer" name="Sample-count" value="%i"/>\n'
                 % node.get("sample_count", 0)
             )
+            handle.write(
+                '    <att type="integer" name="in-db" value="%i"/>\n'
+                % (1 if node.get("in_db", False) else 0)
+            )
             if node["genus"]:
                 handle.write(
                     '    <att type="string" name="Genus" value="%s"/>\n' % node["genus"]
@@ -175,10 +193,12 @@ def main(
     graph_format,
     db_url,
     inputs,
+    method="-",
     min_abundance=100,
     always_show_db=False,
     total_min_abundance=0,
     max_edit_dist=3,
+    ignore_prefixes=None,
     debug=False,
 ):
     """Run the edit-graph command with arguments from the command line.
@@ -198,6 +218,7 @@ def main(
     md5_abundance = Counter()
     md5_sample_count = Counter()
     abundance_by_samples = {}
+    max_sample_abundance = {}
     md5_to_seq = {}
     md5_species = {}
     md5_in_db = set()
@@ -210,20 +231,31 @@ def main(
         sys.exit("If not using -i / --input argument, require -s / --showdb.")
 
     if inputs:
-        if debug:
-            sys.stderr.write("DEBUG: Loading FASTA sequences and abundances\n")
-        for fasta_file in find_requested_files(inputs, ".fasta", debug):
-            # TODO: Refactor this shared code with sample-summary?
-            sample = os.path.basename(fasta_file).rsplit(".", 1)[0]
-            if sample.startswith("Undetermined"):
-                sys.stderr.write("WARNING: Ignoring %s\n" % fasta_file)
-                continue
-            if sample in samples:
-                sys.exit("Duplicate sample name %s" % sample)
+        fasta_files = {}
+        tsv_files = {}
+        if method and method != "-":
+            if debug:
+                sys.stderr.write(f"DEBUG: Loading FASTA and {method} TSV files\n")
+            for fasta_file, tsv_file in find_paired_files(
+                inputs, ".fasta", f".{method}.tsv", ignore_prefixes, debug, strict=True
+            ):
+                sample = file_to_sample_name(fasta_file)
+                fasta_files[sample] = fasta_file
+                tsv_files[sample] = tsv_file
+        else:
+            if debug:
+                sys.stderr.write("DEBUG: Loading FASTA sequences and abundances\n")
+            for fasta_file in find_requested_files(
+                inputs, ".fasta", ignore_prefixes, debug=debug
+            ):
+                sample = file_to_sample_name(fasta_file)
+                fasta_files[sample] = fasta_file
+        for sample, fasta_file in fasta_files.items():
             samples.add(sample)
             with open(fasta_file) as handle:
                 md5_warn = False
                 for title, seq in SimpleFastaParser(handle):
+                    seq = seq.upper()
                     idn, abundance = split_read_name_abundance(title.split(None, 1)[0])
                     md5 = md5seq(seq)
                     if idn != md5:
@@ -232,20 +264,34 @@ def main(
                         continue
                     md5_in_fasta.add(md5)
                     abundance_by_samples[md5, sample] = abundance
+                    max_sample_abundance[md5] = max(
+                        abundance, max_sample_abundance.get(md5, 0)
+                    )
                     md5_abundance[md5] += abundance
                     md5_sample_count[md5] += 1
                     if md5 in md5_to_seq:
-                        assert md5_to_seq[md5] == seq
+                        assert md5_to_seq[md5] == seq, f"{md5} vs {seq}"
                     else:
                         md5_to_seq[md5] = seq
                 if md5_warn:
                     sys.stderr.write(
-                        "WARNING: Sequence(s) in %s not using MD5_abundance naming\n"
-                        % fasta_file
+                        f"WARNING: Sequence(s) in {fasta_file}"
+                        " not using MD5_abundance naming\n"
                     )
+                # Record any assigned species
+                if sample in tsv_files:
+                    for name, _, sp in parse_species_tsv(
+                        tsv_files[sample], min_abundance
+                    ):
+                        if sp:
+                            md5, abundance = split_read_name_abundance(name)
+                            try:
+                                md5_species[md5].update(sp.split(";"))
+                            except KeyError:
+                                md5_species[md5] = set(sp.split(";"))
         sys.stderr.write(
-            "Loaded %i unique sequences from %i FASTA files.\n"
-            % (len(md5_in_fasta), len(samples))
+            f"Loaded {len(md5_in_fasta)} unique sequences"
+            f" from {len(samples)} FASTA files.\n"
         )
         # Drop low total abundance FASTA sequences now (before compute distances)
         if total_min_abundance:
@@ -255,40 +301,46 @@ def main(
                     md5_in_fasta.remove(md5)
                     del md5_to_seq[md5]
             sys.stderr.write(
-                "Minimum total abundance threshold %i "
-                "left %i sequences from FASTA files.\n"
-                % (total_min_abundance, len(md5_in_fasta))
+                f"Minimum total abundance threshold {total_min_abundance}"
+                f" left {len(md5_in_fasta)} sequences from FASTA files.\n"
             )
+        if len(md5_to_seq) > 5000:
+            sys.stderr.write(
+                "WARNING: Over 5000 sequences to plot; aborting edit-graph\n"
+            )
+            # Special return value for use within pipeline
+            return 2
 
     if db_url:
         if debug:
-            sys.stderr.write("DEBUG: Connecting to database %s\n" % db_url)
+            sys.stderr.write(f"DEBUG: Connecting to database {db_url}\n")
         # Connect to the DB,
         Session = connect_to_db(db_url, echo=False)  # echo=debug
         session = Session()
 
-        # Doing a join to pull in the ITS1 and Taxonomy tables too:
+        # Doing a join to pull in the marker and taxonomy tables too:
         cur_tax = aliased(Taxonomy)
-        its1_seq = aliased(ITS1)
+        marker_seq = aliased(RefMarker)
         view = (
             session.query(SequenceSource)
-            .join(its1_seq, SequenceSource.its1)
-            .join(cur_tax, SequenceSource.current_taxonomy)
-            .options(contains_eager(SequenceSource.its1, alias=its1_seq))
-            .options(contains_eager(SequenceSource.current_taxonomy, alias=cur_tax))
+            .join(marker_seq, SequenceSource.marker)
+            .join(cur_tax, SequenceSource.taxonomy)
+            .options(contains_eager(SequenceSource.marker, alias=marker_seq))
+            .options(contains_eager(SequenceSource.taxonomy, alias=cur_tax))
         )
         # Sorting for reproducibility
         view = view.order_by(SequenceSource.id)
         # TODO - Copy genus/species filtering behvaiour from dump command?
 
         for seq_source in view:
-            md5 = seq_source.its1.md5
+            md5 = seq_source.marker.md5
             if not always_show_db and md5 not in md5_in_fasta:
                 # Low abundance or absenst from FASTA files, ignore it
                 continue
-            md5_to_seq[md5] = seq_source.its1.sequence
+            md5_in_db.add(md5)
+            md5_to_seq[md5] = seq_source.marker.sequence
             genus_species = genus_species_name(
-                seq_source.current_taxonomy.genus, seq_source.current_taxonomy.species
+                seq_source.taxonomy.genus, seq_source.taxonomy.species
             )
             try:
                 md5_species[md5].add(genus_species)
@@ -301,33 +353,23 @@ def main(
                 if genus in md5_species[md5]:
                     # If have species level, discard genus level only
                     md5_species[md5].remove(genus)
-        md5_in_db = set(md5_species)
         if always_show_db:
             sys.stderr.write(
-                "Loaded %i unique sequences from database\n" % len(md5_in_db)
+                f"Loaded {len(md5_in_db)} unique sequences from database\n"
             )
         else:
-            sys.stderr.write(
-                "Matched %i unique sequences in database\n" % len(md5_in_db)
-            )
+            sys.stderr.write(f"Matched {len(md5_in_db)} unique sequences in database\n")
 
     if db_url and inputs and always_show_db:
         sys.stderr.write(
-            "DB had %i sequences (%i not in FASTA), "
-            "FASTA had %i sequences (%i not in DB).\n"
-            % (
-                len(md5_in_db),
-                len(md5_in_db.difference(md5_in_fasta)),
-                len(md5_in_fasta),
-                len(md5_in_fasta.difference(md5_in_db)),
-            )
+            f"DB had {len(md5_in_db)} sequences"
+            f" ({len(md5_in_db.difference(md5_in_fasta))} not in FASTA),"
+            f" FASTA had {len(md5_in_fasta)} sequences"
+            f" ({len(md5_in_fasta.difference(md5_in_db))} not in DB).\n"
         )
         sys.stderr.write(
-            "DB and FASTA had %i sequences in common; %i combined.\n"
-            % (
-                len(md5_in_db.intersection(md5_in_fasta)),
-                len(md5_in_db.union(md5_in_fasta)),
-            )
+            f"DB and FASTA had {len(md5_in_db.intersection(md5_in_fasta))} sequences"
+            f" in common; {len(md5_in_db.union(md5_in_fasta))} combined.\n"
         )
 
     if not md5_to_seq:
@@ -336,7 +378,7 @@ def main(
     # For drawing performance reasons, calculate the distances, and then may
     # drop nodes with no edges (unless for example DB entry at species level,
     # or for environmental sequences at high abundance)
-    md5_list = list(md5_to_seq)
+    md5_list = sorted(md5_to_seq)
     wanted = set()
     n = len(md5_list)
     todo = n * (n - 1) // 2
@@ -351,22 +393,56 @@ def main(
                 done += 1
                 if debug and done % 100000 == 0:
                     print(
-                        "DEBUG: Computed %i of %i Levenshtein edit distances (%i%%)"
-                        % (done, todo, int(done * 100.0 / todo))
+                        f"DEBUG: Computed {done} of {todo} Levenshtein edit distances"
+                        f"({int(done * 100.0 / todo)}%)"
                     )
                 if d and d <= max_edit_dist:
                     wanted.add(check1)
                     wanted.add(check2)
-    assert done == todo, "%r vs %s" % (done, todo)
+    assert done == todo, f"{done!r} vs {todo}"
     sys.stderr.write(
-        "Computed %i Levenshtein edit distances between %i sequences.\n" % (done, n)
+        f"Computed {done} Levenshtein edit distances between {n} sequences.\n"
     )
     sys.stderr.write(
-        "Will draw %i nodes with at least one edge (%i are isolated sequences).\n"
-        % (len(wanted), n - len(wanted))
+        f"Will draw {len(wanted)} nodes with at least one edge"
+        f" ({n - len(wanted)} are isolated sequences).\n"
     )
     del done, todo
 
+    for md5 in md5_list:
+        if md5 not in wanted:
+            # Will include high abundance singletons too
+            if total_min_abundance <= md5_abundance.get(md5, 0):
+                wanted.add(md5)
+    if inputs:
+        sys.stderr.write(
+            "Including high abundance isolated sequences,"
+            f" will draw {len(wanted)} nodes.\n"
+        )
+
+    if graph_format == "matrix":
+        if graph_output == "-":
+            handle = sys.stdout
+        else:
+            handle = open(graph_output, "w")
+        cols = "\t".join(check2 for check2 in md5_list if check2 in wanted)
+        handle.write(f"MD5\tSpecies\t{cols}\n")
+        del cols
+        for i, check1 in enumerate(md5_list):
+            if check1 not in wanted:
+                continue
+            sp = ",".join(sorted(md5_species.get(check1, [])))
+            dists = "\t".join(
+                str(distances[i, j])
+                for j, check2 in enumerate(md5_list)
+                if check2 in wanted
+            )
+            handle.write(f"{check1}\t{sp}\t{dists}\n")
+            del sp, dists
+        if graph_output != "-":
+            handle.close()
+        return 0
+
     # Matrix computation of multi-step paths vs edit distances, e.g.
     # will use fact A-B is 1bp and B-C is 2bp to skip drawing A-C of 3bp.
     one_bp = distances == 1  # boolean
@@ -377,17 +453,6 @@ def main(
     )
     del one_bp, two_bp
 
-    for md5 in md5_list:
-        if md5 not in wanted:
-            # Will include high abundance singletons too
-            if total_min_abundance <= md5_abundance.get(md5, 0):
-                wanted.add(md5)
-    if inputs:
-        sys.stderr.write(
-            "Including high abundance isolated sequences, will draw %i nodes.\n"
-            % len(wanted)
-        )
-
     if md5_sample_count:
         # scaling factor
         SIZE = 100.0 / max(md5_sample_count.values())
@@ -407,7 +472,7 @@ def main(
             continue
         sp = md5_species.get(md5, [])
         genus = sorted({_.split(None, 1)[0] for _ in sp})
-        if not genus:
+        if md5 not in md5_in_db or not genus:
             node_color = "#808080"  # grey
         elif len(genus) > 1:
             node_color = "#FF8C00"  # dark orange
@@ -415,24 +480,26 @@ def main(
             node_color = genus_color[genus[0]]
         else:
             node_color = "#8B0000"  # dark red
-        if any(species_level(_) for _ in sp):
+        if sp:
+            # TODO - Remove this Phytophthora specific hack, or automate it?
             node_label = "\n".join(sorted(sp)).replace("Phytophthora", "P.")
         else:
-            # Genus only
+            # No species, not even genus only - fall back on MD5 as node ID
             node_label = ""
         genus = ";".join(sorted({_.split(None, 1)[0] for _ in sp}))
-        # DB only entries get size one, FASTA entries can be up to 100.
-        abundance = md5_abundance.get(md5, 0)
+        # DB only entries get size zero, FASTA entries can be up to 100.
         node_size = max(1, SIZE * md5_sample_count.get(md5, 0))
         G.add_node(
             md5,
             color=node_color,
             size=node_size,
             label=node_label,
-            abundance=abundance,
+            total_abundance=md5_abundance.get(md5, 0),
+            max_sample_abundance=max_sample_abundance.get(md5, 0),
             sample_count=md5_sample_count.get(md5, 0),
             genus=genus,
             taxonomy=";".join(sorted(sp)),
+            in_db=md5 in md5_in_db,
         )
 
     edge_count = 0
@@ -496,7 +563,7 @@ def main(
                     check1,
                     check2,
                     len=edge_length,
-                    edit_dist=dist,
+                    edit_dist=int(dist),  # cast numpy.uint64 to int
                     K=edge_length,
                     weight=edge_weight,
                     style=edge_style,
@@ -506,17 +573,15 @@ def main(
 
     if debug:
         sys.stderr.write(
-            "DEBUG: %i edges up to maximum edit distance %i\n"
-            % (edge_count, max_edit_dist)
+            f"DEBUG: {edge_count} edges up to maximum edit distance"
+            f" {max_edit_dist}\n"
         )
         sys.stderr.write(
-            "DEBUG: %i one-bp edges; %i two-bp edges; %i three-bp edges.\n"
-            % (edge_count1, edge_count2, edge_count3)
+            f"DEBUG: {edge_count1} one-bp edges; {edge_count2} two-bp edges;"
+            " {edge_count3} three-bp edges.\n"
         )
         assert edge_count == edge_count1 + edge_count2 + edge_count3
-        sys.stderr.write(
-            "DEBUG: Dropped %i redundant 2-bp or 3-bp edges.\n" % redundant
-        )
+        sys.stderr.write(f"DEBUG: Dropped {redundant} redundant 2-bp or 3-bp edges.\n")
 
     render = {
         "gexf": nx.write_gexf,
@@ -529,7 +594,7 @@ def main(
         write_fn = render[graph_format]
     except KeyError:
         # Typically this would be caught in __main__.py
-        sys.exit("ERROR: Unexpected graph output format: %s" % graph_format)
+        sys.exit(f"ERROR: Unexpected graph output format: {graph_format}")
 
     write_fn(G, "/dev/stdout" if graph_output == "-" else graph_output)
 
